@@ -9,7 +9,8 @@ setRefClass("OAuth",
               verifier = "character",
               requestURL = "character",
               authURL = "character",
-              accessURL = "character"
+              accessURL = "character",
+              signMethod = 'character'
               ),
             methods = list(
               initialize = function(needsVerifier, ...) {
@@ -19,18 +20,21 @@ setRefClass("OAuth",
                   needsVerifier <<- TRUE
                 handshakeComplete <<- FALSE
                 callSuper(...)
+                .self
               },
-              handshake = function() {
+              
+              handshake = function(signMethod='HMAC', curl=getCurlHandle(), ...) {
                 ' Performs the OAuth handshake.  In most cases
                   the user will need to complete a manual step
                   with their web browser, entering a PIN into
                   this function.
                 '
                 handshakeComplete <<- FALSE
-                resp <- oauthPOST(requestURL,
-                                  consumerKey,
-                                  consumerSecret,
-                                  NULL, NULL)
+                signMethod <<- signMethod
+                resp <- oauthPOST(.self$requestURL, 
+                                  .self$consumerKey,
+                                  .self$consumerSecret,
+                                  NULL, NULL, signMethod=.self$signMethod, curl=curl, ...)
                 vals <- parseResponse(resp)
                 if (!all(c('oauth_token', 'oauth_token_secret') %in%
                          names(vals))) {
@@ -40,8 +44,8 @@ setRefClass("OAuth",
                 }
                 oauthKey <<- vals['oauth_token']
                 oauthSecret <<- vals['oauth_token_secret']
-                if (needsVerifier) {
-                  verifyURL <- paste(authURL, "?oauth_token=",
+                if (.self$needsVerifier) {
+                  verifyURL <- paste(.self$authURL, "?oauth_token=",
                                      oauthKey, sep='')
                   msg <- paste("To enable the connection, please direct",
                                " your web browser to: \n",
@@ -49,15 +53,11 @@ setRefClass("OAuth",
                                "\nWhen complete, record the PIN given ",
                                "to you and provide it here: ", sep='')
                   verifier <<- readline(prompt=msg)
-                  accessURL <<- paste(accessURL,
-                                      "?oauth_verifier=",
-                                      verifier, sep='')
                 }
-
-                resp <- oauthPOST(accessURL,
-                                  consumerKey,
-                                  consumerSecret,
-                                  oauthKey, oauthSecret)
+                params <- c(oauth_verifier=.self$verifier)
+                resp <- oauthPOST(.self$accessURL, .self$consumerKey, .self$consumerSecret,
+                                  .self$oauthKey, .self$oauthSecret, signMethod=.self$signMethod,
+                                  curl=getCurlHandle(), params=params, ...)
                 vals <- parseResponse(resp)
                 if (!all(c('oauth_token', 'oauth_token_secret') %in%
                          names(vals))) {
@@ -75,18 +75,19 @@ setRefClass("OAuth",
                 'Will report if this object is verified or not.
                  Verification can either involve not needing it
                  in the first place, or as part of the handshake'
-                if (needsVerifier)
+                if (.self$needsVerifier)
                   length(verifier) != 0
                 else
                   TRUE
               },
               
-              OAuthRequest = function(URL, method="GET", customHeader=NULL) {
+              OAuthRequest = function(URL, params=character(), method="GET",
+                                      customHeader=NULL, curl=getCurlHandle(), ...) {
                 ' If the OAuth handshake has been completed, will
                 submit a URL request with an OAuth signature, returning
                 any response from the server
                 '
-                if (! handshakeComplete)
+                if (! .self$handshakeComplete)
                   stop("This OAuth instance has not been verified")
 
                 httpFunc <- switch(method,
@@ -94,26 +95,16 @@ setRefClass("OAuth",
                                    GET = oauthGET,
                                    stop("method must be POST or GET"))
 
-                httpFunc(URLencode(URL), consumerKey, consumerSecret,
-                         oauthKey, oauthSecret, customHeader)
+                httpFunc(URLencode(URL), params=params, consumerKey=.self$consumerKey,
+                         consumerSecret=.self$consumerSecret,
+                         oauthKey=.self$oauthKey, oauthSecret=.self$oauthSecret,
+                         customHeader.self$customHeader, curl=getCurlHandle(), signMethod=.self$signMethod, ...)
               }
               )
             )
 
-
-oauthPOST <- function(url, consumerKey, consumerSecret,
-                      oauthKey, oauthSecret,
-                      customHeader=NULL) {
-  .Call("ROAuth_POST", url, consumerKey, consumerSecret,
-        oauthKey, oauthSecret, customHeader, PACKAGE="ROAuth")
-}
-
-oauthGET <- function(url, consumerKey, consumerSecret,
-                     oauthKey, oauthSecret,
-                     customHeader=NULL) {
-  .Call("ROAuth_GET", url, consumerKey, consumerSecret,
-        oauthKey, oauthSecret, customHeader, PACKAGE="ROAuth")
-}
+OAuthFactory <- getRefClass("OAuth")
+OAuthFactory$accessors(names(OAuthFactory$fields()))
 
 
 parseResponse <- function(response) {
@@ -128,3 +119,46 @@ parseResponse <- function(response) {
   out
 }
 
+oauthPOST <- function(url, consumerKey, consumerSecret,
+                      oauthKey, oauthSecret, params=character(), customHeader = NULL,
+                      curl = getCurlHandle(), signMethod='HMAC', ...) {
+  if(is.null(curl))
+    curl <- getCurlHandle()
+  
+  auth <- signRequest(url, params, consumerKey, consumerSecret,
+                      oauthKey=oauthKey, oauthSecret=oauthSecret,
+                      httpMethod="POST", signMethod=signMethod)
+  opts <- list(...)
+  
+  ## post ,specify the method
+  ## We should be able to use postForm() but we have to work out the issues
+  ## with escaping, etc. to match the signature mechanism.
+  if(TRUE) {   
+    reader <- dynCurlReader(curl, baseURL = url, verbose = FALSE)
+    fields <- paste(names(auth), sapply(auth, curlPercentEncode),
+                    sep = "=", collapse = "&")
+    curlPerform(curl = curl, URL = url, postfields = fields,
+                writefunction = reader$update, ...)
+    reader$value()
+  } else
+  postForm(url, .params = c(params, lapply(auth, I)), curl = curl,
+           .opts = opts, style = "POST")
+}
+
+#XXX? use .opts for the curl options.
+#     add a ... for the parameters.
+
+oauthGET <- function(url, consumerKey, consumerSecret,
+                     oauthKey, oauthSecret, params=character(), customHeader = NULL,
+                     curl = getCurlHandle(), signMethod='HMAC', ..., .opts = list(...)) {
+  ##   opts = list(httpheader = c(Authentication = paste(names(auth),  auth, sep = "=", collapse = "\n   ")), ...)
+  if(is.null(curl))
+    curl <- getCurlHandle()
+   
+   auth <- signRequest(url, params, consumerKey, consumerSecret,
+                       oauthKey=oauthKey, oauthSecret=oauthSecret,
+                       httpMethod="GET", signMethod=signMethod)
+
+   params <- c(params, as.list(auth))
+   getForm(url, .params = params, curl = curl, .opts = c(httpget = TRUE,  list(...)))
+}
